/*
 * Minecraft Forge
 * Copyright (c) 2016-2019.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version 2.1
 * of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package net.minecraftforge.fml.client.gui;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.Gui;
import net.minecraft.client.gui.GuiListExtended;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.versions.forge.ForgeVersion;
import net.minecraftforge.fml.MavenVersionStringHelper;
import net.minecraftforge.fml.VersionChecker;
import net.minecraftforge.fml.loading.moddiscovery.ModInfo;

import static net.minecraft.util.StringUtils.stripControlCodes;

/**
 * @author cpw
 *
 */
public class GuiSlotModList extends GuiListExtended<GuiSlotModList.ModEntry>
{
    private static final ResourceLocation VERSION_CHECK_ICONS = new ResourceLocation(ForgeVersion.MOD_ID, "textures/gui/version_check_icons.png");
    private final int listWidth;

    private GuiModList parent;

    public GuiSlotModList(GuiModList parent, int listWidth)
    {
        super(parent.getMinecraftInstance(), listWidth, parent.field_146295_m, 32, parent.field_146295_m - 67 + 4, parent.getFontRenderer().field_78288_b * 2 + 8);
        this.parent = parent;
        this.listWidth = listWidth;
        this.refreshList();
    }

    @Override
    protected int func_148137_d()
    {
        return this.listWidth;
    }

    @Override
    public int func_148139_c()
    {
        return this.listWidth;
    }

    void refreshList() {
        this.func_195086_c();
        parent.buildModList(this::func_195085_a, mod->new ModEntry(mod, this.parent));
    }

    @Override
    protected boolean func_148131_a(int index)
    {
        return this.parent.modIndexSelected(index);
    }

    @Override
    protected void func_148123_a()
    {
        this.parent.func_146276_q_();
    }

    class ModEntry extends GuiListExtended.IGuiListEntry<ModEntry> {
        private final ModInfo modInfo;
        private final GuiModList parent;

        ModEntry(ModInfo info, GuiModList parent) {
            this.modInfo = info;
            this.parent = parent;
        }

        @Override
        public void func_194999_a(int entryWidth, int entryHeight, int mouseX, int mouseY, boolean p_194999_5_, float partialTicks)
        {
            int top = this.func_195001_c();
            int left = this.func_195002_d();
            String name = func_76338_a(modInfo.getDisplayName());
            String version = func_76338_a(MavenVersionStringHelper.artifactVersionToString(modInfo.getVersion()));
            VersionChecker.CheckResult vercheck = VersionChecker.getResult(modInfo);
            FontRenderer font = this.parent.getFontRenderer();
            font.func_211126_b(font.func_78269_a(name, listWidth),left + 3, top + 2, 0xFFFFFF);
            font.func_211126_b(font.func_78269_a(version, listWidth), left + 3 , top + 2 + font.field_78288_b, 0xCCCCCC);
            if (vercheck.status.shouldDraw())
            {
                //TODO: Consider adding more icons for visualization
                Minecraft.func_71410_x().func_110434_K().func_110577_a(VERSION_CHECK_ICONS);
                GlStateManager.func_179131_c(1, 1, 1, 1);
                GlStateManager.func_179094_E();
                Gui.func_146110_a(field_148151_d - (field_148158_l / 2 + 4), GuiSlotModList.this.field_148153_b + (field_148158_l / 2 - 4), vercheck.status.getSheetOffset() * 8, (vercheck.status.isAnimated() && ((System.currentTimeMillis() / 800 & 1)) == 1) ? 8 : 0, 8, 8, 64, 16);
                GlStateManager.func_179121_F();
            }
        }

        @Override
        public boolean mouseClicked(double p_mouseClicked_1_, double p_mouseClicked_3_, int p_mouseClicked_5_)
        {
            parent.selectModIndex(this.func_195003_b());
            return false;
        }
    }
}
